<?php

function genie_refresher_warmup_dist($t) {
	include_spip("inc/refresher_functions");
	
	$warmup_start_time = lire_config('refresher/warmup_start_time');
	// if no start time defined we don't run it
	if($warmup_start_time != 0 && $warmup_start_time != ''){
		// first we allow this process only if there is no recalcul job waiting (or scheduled for the next minute) to avoid competition
		$res = sql_select("id","refresher_queue","date<=DATE_ADD(NOW(), INTERVAL 1 MINUTE)","","id",1);
		if($row = sql_fetch($res)){ 
			spip_log("refresher comming soon -> we cancel warmup", "refresher_warmup"._LOG_INFO_IMPORTANTE);
			return 1;
		}
		$links = array();
		$t = time();
		$n = 0;
		$warmup_start_time = lire_config('refresher/warmup_start_time');
		$res = sql_select("uri","refresher_warmup","statut='pending'","","id",_WARMUP_PROCESS_NB);
		spip_log("warmup start", "refresher_warmup"._LOG_INFO_IMPORTANTE);
		if(sql_count($res) == 0){ 
			ecrire_config("/meta/refresher/warmup_start_time", 0);
			spip_query("TRUNCATE TABLE refresher_warmup");
		}
		while($row = sql_fetch($res)){
			// if server load high, we leave
			$instant_load = 1;
			if(function_exists('sys_getloadavg')){ 
				$load = sys_getloadavg();
				$instant_load = $load[0];
			}
			if(defined('_REFRESHER_MAX_LOAD') && $instant_load > _REFRESHER_MAX_LOAD){ 
				spip_log("warmup interrupted (n=".$n."): Server load too high", "refresher_warmup"._LOG_INFO_IMPORTANTE);
				return;
			}
			// if we already took a minute, we leave
			if(defined('_WARMUP_MAX_TIME') && (time() - $t > _WARMUP_MAX_TIME)){ 
				spip_log("warmup interrupted (n=".$n."): timeout", "refresher_warmup"._LOG_INFO_IMPORTANTE);
				return;
			}
			// if mysql unresponsive, we leave
			if(!spip_connect()){ 
				spip_log("warmup interrupted (n=".$n."): MYSQL unresponsive", "refresher_warmup"._LOG_INFO_IMPORTANTE);
				return;
			}
			// if too many mysql processes, we leave
			$res_proc = spip_query('show processlist');
			if(defined('_REFRESHER_MAX_SQL_PROCESSES') && sql_count($res_proc) > _REFRESHER_MAX_SQL_PROCESSES){ 
				spip_log("warmup interrupted (n=".$n."): Too many MYSQL processes (".sql_count($res_proc).")", "refresher_warmup"._LOG_INFO_IMPORTANTE);
				return;
			}
			// if everything ok so far, we send http requests
			// if URL has been refreshed somehow since cache warmup start, we skip it
			$parallel_uris = array();
			$parallel_urls = array();
			$process_url = true;
			$res2 = sql_select("maj","refresher_urls","uri=".sql_quote($row['uri']),"","",1);
			if($row2 = sql_fetch($res2)){
				$date_obj = date_create_from_format('Y-m-d H:i:s', $row2["maj"]);
				$date_tmp = date_format($date_obj, 'U');
				if($date_tmp > $warmup_start_time){ 
					sql_updateq("refresher_warmup", array("statut" => "done"), "uri=".sql_quote($row['uri']));
					$process_url = false;
				}
			}
			$n++;
			if($process_url){ 
				spip_log("warmup processing URI (n=".$n."): ".$GLOBALS['meta']['adresse_site'].'/'.$row['uri'], "refresher_warmup"._LOG_INFO_IMPORTANTE);
				$parallel_uris[] = $row['uri'];
				$parallel_urls[] = $GLOBALS['meta']['adresse_site'].'/'.$row['uri'];
			}
			else spip_log("warmup no need processing URI (n=".$n."): ".$GLOBALS['meta']['adresse_site'].'/'.$row['uri'], "refresher_warmup"._LOG_INFO_IMPORTANTE);
			for($i = 1; $i < _WARMUP_PARALLEL_NB; $i ++){
				if($row = sql_fetch($res)){
					// if URL has been refreshed somehow since cache warmup start, we skip it
					$process_url = true;
					$res2 = sql_select("maj","refresher_urls","uri=".sql_quote($row['uri']),"","",1);
					if($row2 = sql_fetch($res2)){
						$date_obj = date_create_from_format('Y-m-d H:i:s', $row2["maj"]);
						$date_tmp = date_format($date_obj, 'U');
						if($date_tmp > $warmup_start_time){ 
							sql_updateq("refresher_warmup", array("statut" => "done"), "uri=".sql_quote($row['uri']));
							$process_url = false;
						}
					}
					if($process_url){
						spip_log("warmup processing URI (n=".$n."): ".$GLOBALS['meta']['adresse_site'].'/'.$row['uri'], "refresher_warmup"._LOG_INFO_IMPORTANTE);
						$parallel_uris[] = $row['uri'];
						$parallel_urls[] = $GLOBALS['meta']['adresse_site'].'/'.$row['uri'];
					}
					else spip_log("warmup no need processing URI (n=".$n."): ".$GLOBALS['meta']['adresse_site'].'/'.$row['uri'], "refresher_warmup"._LOG_INFO_IMPORTANTE);
				}
			}
			
			if(lire_config('refresher/warmup_recalcul') == 'yes') $links = refresh_url($parallel_urls, array(), 'warmup');
			else $links = access_url($parallel_urls, array(), 'warmup');
			foreach($parallel_uris as $uri) sql_updateq("refresher_warmup", array("statut" => "done"), "uri=".sql_quote($uri));
			
			foreach($links as $link){
				if(strpos($link["href"], $GLOBALS["meta"]["adresse_site"]) !== false || strpos($link["href"], "://") === false){
					if(substr($link["href"], 0, 1) != "#"){
						$test_link = str_replace($GLOBALS["meta"]["adresse_site"], "", $link["href"]);
						if(substr($test_link, 0, 1) == "/") $test_link = substr($test_link, 1);
						$arr = explode("?", $test_link);
						$test_link = $arr[0];
						$arr = explode("#", $test_link);
						$test_link = $arr[0];
						$res2 = sql_select("id, statut","refresher_warmup","uri=".sql_quote($test_link),"","",1);
						if($row2 = sql_fetch($res2)){}
						else{ 
							// if already refreshed recently on regular refresher we add it already done
							$statut = "pending";
							$res2 = sql_select("maj","refresher_urls","uri=".sql_quote($test_link),"","",1);
							if($row2 = sql_fetch($res2)){ 
								$date_obj = date_create_from_format('Y-m-d H:i:s', $row2["maj"]);
								$date_tmp = date_format($date_obj, 'U');
								if($date_tmp > $warmup_start_time) $statut = "done";
							}
							sql_insertq("refresher_warmup", array("uri" => $test_link, "statut" => $statut));
						}
					}
				}
			}
		}
		spip_log("warmup end", "refresher_warmup"._LOG_INFO_IMPORTANTE);
	}
	return 1;
}

?>